"""Dictionary exercises"""


def count_words():
    """Return the number of times each word occurs in the string."""


def group_by():
    """Return dictionary of items grouped by given key function."""


def get_shared_keys():
    """Return the keys that are shared by both dictionaries."""


def dict_from_truple():
    """Turn three-item tuples into a dictionary of two-valued tuples."""


words = {
    'esta': 'is',
    'la': 'the',
    'en': 'in',
    'gato': 'cat',
    'casa': 'house',
    'el': 'the',
}


def translate():
    """Return a transliterated version of the given sentence."""


def pluck():
    """Deep-query a dictionary"""


def get_all_factors():
    """Return a dictionary mapping numbers to their factors."""


def get_ascii_codes():
    """Return a dictionary mapping the strings to ASCII codes."""


def dict_from_tuple():
    """Turn multi-item tuples into a dictionary of two-valued tuples."""


def flip_dict():
    """Return a new dictionary that maps the original values to the keys."""


def sort_dict(old_dict):
    """Return a new dictionary that is sorted."""
